import logging
logger = logging.getLogger('archetypes.memberdatastorage')
logger.setLevel(logging.DEBUG)

from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.Archetypes import atapi
from Products.Archetypes.Registry import registerField
from archetypes.memberdatastorage.memberdatastorage import MemberdataStorage


class MemberPropertyField(atapi.StringField):
    """Making string-valued member properties managed by the memberdata 
       tool available in content space.

       Uses the Memberdata storage layer.

       Overrides the getDefault method to look up existing
       member properties first.

       Per default the field is mapped to a member property
       with the same id as the field's name. To connect to
       a differently named member property, this needs to be 
       specified twice: (i) by setting the 'member_property_id'
       on the field and (ii) when setting the storage. 
    """

    _properties = atapi.StringField._properties.copy()
    _properties.update({
        'member_property_id' : None,
        'storage' : MemberdataStorage(),   # pass in the member property id
                                           # here if mapping to a property
                                           # named differently than the field
        })

    security  = ClassSecurityInfo()

    security.declarePublic('getDefault')
    def getDefault(self, instance):
        """Return the default value to be used for initializing this
        field. First hunts for a matching member property before
        doing the regular stuff."""
        property = self._huntProperty(instance)
        return property or atapi.StringField.getDefault(self, instance)

    def _huntProperty(self, instance):
        logger.debug("hunting for property %s" % self.member_property_id)
        property_id = self.member_property_id or self.getName()
        content_id = instance.getId()
        membership_tool = getToolByName(instance, 'portal_membership')
        member = membership_tool.getMemberById(content_id)
        if member is None:
            return None
        return member.getProperty(property_id)


registerField(MemberPropertyField,
              title = 'Member property field',
              description = 'Used to manage a member property '
                            'from a content object.')



import logging
logger = logging.getLogger('archetypes.memberdatastorage')
logger.setLevel(logging.DEBUG)

from Products.CMFCore.utils import getToolByName
from Products.Archetypes.Storage import StorageLayer
from Products.Archetypes.Storage.annotation import AnnotationStorage
from Products.Archetypes.interfaces.storage import IStorage
from Products.Archetypes.interfaces.layer import ILayer
from Products.Archetypes.Field import encode

from AccessControl import ClassSecurityInfo
from Products.Archetypes.Registry import registerStorage

class MemberdataStorage(AnnotationStorage, StorageLayer):
    """A facade storage which delegates to
    the site's Memberdata Tool for actually
    storing content item field values as
    memberdata values. 

    This heavily relies on name magic.
    It is assumed that the correponding member
    and content instances have *exactly* the
    same id. Furthermore the field names have
    to match corresponding memberdata properties
    unless the memberproperty id is specified when
    instanciating the storage.

    For safety reasons values are always stored in
    AnnotationStorage (as well) and it automatically
    falls back to this if no associated member can be
    found.
    """

    security = ClassSecurityInfo()

    __implements__ = (IStorage, ILayer)

    
    def __init__(self, property_id='', migrate=False):
        self._member_property_id = property_id
        self._migrate = migrate
        AnnotationStorage.__init__(self, migrate)

    
    security.declarePrivate('getMembershipTool')
    def getMembershipTool(self, instance):
        return getToolByName(instance, 'portal_membership')

    security.declarePrivate('initializeInstance')
    def initializeInstance(self, instance, item=None, container=None):
        pass

    security.declarePrivate('initializeField')
    def initializeField(self, instance, field):
        name = self._member_property_id or field.getName()
        logger.debug("Initializing memberproperty field '%s' to look for %s" % \
            (field.getName(), name))
        membership_tool = self.getMembershipTool(instance)
        member = membership_tool.getMemberById(instance.getId())
        memberdata_tool = getToolByName(instance, 'portal_memberdata')
        if not memberdata_tool.hasProperty(name):
            logger.debug("Adding %s property to the memberdata tool" % name)
            memberdata_tool.manage_addProperty(name, field.default, field.type)
        if member is not None:
            value = member.getProperty(name)
            self.set(name, instance, value)

    security.declarePrivate('get')
    def get(self, name, instance, **kwargs):
        membership_tool = self.getMembershipTool(instance)
        member = membership_tool.getMemberById(instance.getId())
        if member is not None:
            id = self._member_property_id or name
            value = member.getProperty(id)
            return value
        else:
            return AnnotationStorage.get(self, name, instance, **kwargs)

    security.declarePrivate('set')
    def set(self, name, instance, value, **kwargs):
        membership_tool = self.getMembershipTool(instance)
        member = membership_tool.getMemberById(instance.getId())
        if type(value) == type(u''):
            value = encode(value, instance)
        if member is not None:
            id = self._member_property_id or name
            member.setMemberProperties({id:value})
        # yes, set it in two places so we can fall back to it
        AnnotationStorage.set(self, name, instance, value, **kwargs)


    security.declarePrivate('unset')
    def unset(self, name, instance, **kwargs):
        pass

    security.declarePrivate('cleanupField')
    def cleanupField(self, instance, field, **kwargs):
        pass

    security.declarePrivate('cleanupInstance')
    def cleanupInstance(self, instance, item=None, container=None):
        pass

registerStorage(MemberdataStorage)



"""This is an integration "unit" test. It uses PloneTestCase because I'm lazy
"""


import unittest

from Products.CMFCore.utils import getToolByName

from Products.PloneTestCase import PloneTestCase as ptc

ptc.setupPloneSite()


class TestMemberPropertyField(ptc.PloneTestCase):

    """Test usage of the member property field.
    """

    def afterSetUp(self):

        """This method is called before each single test. It can be used to
        set up common state. Setup that is specific to a particular test 
        should be done in that test method.
        """

        self.membership = getToolByName(self.portal, 'portal_membership')

        # create our own member to mess with
        self.membership.addMember('user1', 'u1', ['Member'], [],
                                  {'email': 'user1@host.com',
                                   'fullname': 'User 1'})

        self.folder.invokeFactory('Document', 'my-doc')
        self.folder.invokeFactory('Document', 'user1')

        from archetypes.memberdatastorage.memberpropertyfield \
            import MemberPropertyField
        
        self.field_id = 'fullname'
        self.initial_value = 'Monty Python'

        for id in ['my-doc', 'user1']:
            page = self.folder[id]
            page.schema.addField(MemberPropertyField(self.field_id))
            page.schema[self.field_id].set(page,self.initial_value)


    def beforeTearDown(self):

        """This method is called after each single test. It can be used for
        cleanup, if you need it. Note that the test framework will roll back
        the Zope transaction at the end of each test, so tests are generally
        independent of one another. However, if you are modifying external
        resources (say a database) or globals (such as registering a new
        adapter in the Component Architecture during a test), you may want to
        tear things down here.
        """


    def test_fallback_use_of_memberpropertyfield(self):

        page = self.folder['my-doc']
        value = page.schema[self.field_id].get(page)
        self.assertEquals(self.initial_value, value)


    def test_default_use_of_memberpropertyfield(self):

        member = self.membership.getMemberById('user1')
        member_value = member.getProperty(self.field_id)

        self.assertEquals(self.initial_value, member_value)

        # and now the other way around
        new_name = 'Buster Keaton'
        member.setMemberProperties({'fullname':new_name})
        page = self.folder['user1']
        content_value = page.schema[self.field_id].get(page)
        
        self.assertEquals(content_value, new_name)


    def test_memberproperty_creation(self):
        
        new_field_id = 'age'
        member = self.membership.getMemberById('user1')
        
        # Assert user doesn't have the property yet
        self.failIf(member.hasProperty('age'))      

        page = self.folder['user1']

        from archetypes.memberdatastorage.memberpropertyfield \
            import MemberPropertyField
        
        page.schema.addField(MemberPropertyField('age',default='youngster'))
        page.schema.initializeLayers(page)  # to trigger the field's initializeLayer
        
        # re-get the member to pick up the update
        member = self.membership.getMemberById('user1')
        
        # Assert member does have the property now
        self.failUnless(member.hasProperty('age')) 
        
        property_value = member.getProperty('age')
        content_value  = page.schema['age'].get(page)
        
        self.assertEquals(property_value, content_value)
        

    ## TODO: the fallback could be tested more rigorously; 
    ## test properties other than string type

def test_suite():

    """This sets up a test suite that actually runs the tests in the class
    above
    """

    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestMemberPropertyField))
    return suite
